using System;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.IO;	
using System.ComponentModel;
using System.Collections;
using System.Collections.Specialized;
using System.Text.RegularExpressions;

namespace TreeViewExamples
{
	/// <summary>
	///		Summary description for CodeViewer.
	/// </summary>
	public abstract class CodeViewer : System.Web.UI.UserControl
	{
		private const string tabDataDiv = "<div style='display:none' id='{0}TabContent'>{1}</div>";
		private const string tabDiv = "<td align='center' valign='top' id='{0}Tab' onclick=\"displayTab('{0}'); return false\" class='tab'><img src='Img/codeTabsArrowDisabled.gif'><img src='Img/codeTabs{0}.gif' border='0'></td>";

		private bool _description = false;
		private bool _ascx = false;
		private bool _xml = false;
		private bool _css = false;
		private bool _cs = false;
		private bool _ascxCs = false;
		private bool _vb = false;
		private string _path = "";
		private string _fileName = "";
		private string _defaultTab = "Description";
		private string _imageDir = string.Empty;

		private string _xmlContent = string.Empty;
		private string _ascxContent = string.Empty;
		private string _cssContent = string.Empty;

		public void setXmlContent (string xml)
		{
			_xmlContent = xml;			
		}

		public void setAscxContent (string ascx)
		{
			_ascxContent = ascx;
		}

		public void setCssContent (string css)
		{
			_cssContent = css;
		}

		protected PlaceHolder tabPlaceholder;
		protected PlaceHolder codePlaceholder;

		//RadViewer shits
		public Unit FontSize = Unit.Point(8);
		public String FontName = "Tahoma";
		public Int32 TabSpaces = 3;
		public bool ReplaceTabs = true;
		const string TAG_FNTRED		= "<font color=\"red\">";
		const string TAG_FNTBLUE	= "<font color=\"blue\">";
		const string TAG_FNTGRN		= "<font color=\"green\">";
		const string TAG_FNTMRN		= "<font color=\"maroon\">";
		const string TAG_EFONT		= "</font>" ;
		private string _applicationPath = "";
		private string TabString = "";

		private void Page_Load(object sender, System.EventArgs e){}

		private void OnPreRender(object sender, System.EventArgs e)
		{
			if (_description)
			{
				tabPlaceholder.Controls.Add(new LiteralControl(String.Format(tabDataDiv, "Description", this.RenderHtmlFile(this.Path + this.FileName + ".html"))));
				codePlaceholder.Controls.Add(new LiteralControl(String.Format(tabDiv, "Description")));
			}

			if (_ascxContent != string.Empty)
			{
				tabPlaceholder.Controls.Add(new LiteralControl(String.Format(tabDataDiv, "Ascx", this.RenderContent ( new StringReader(_ascxContent),".ascx" ) )));
				codePlaceholder.Controls.Add(new LiteralControl(String.Format(tabDiv, "Ascx")));
			}
			else if (_ascx)
			{
				tabPlaceholder.Controls.Add(new LiteralControl(String.Format(tabDataDiv, "Ascx", this.Render(this.Path + this.FileName + ".ascx"))));
				codePlaceholder.Controls.Add(new LiteralControl(String.Format(tabDiv, "Ascx")));
			}

			if (_xmlContent != string.Empty)
			{
				tabPlaceholder.Controls.Add(new LiteralControl(String.Format(tabDataDiv, "Xml", this.RenderContent(new StringReader(_xmlContent), ".xml"))));
				codePlaceholder.Controls.Add(new LiteralControl(String.Format(tabDiv, "Xml")));
			}
			else if (_xml)
			{
				tabPlaceholder.Controls.Add(new LiteralControl(String.Format(tabDataDiv, "Xml", this.Render(this.Path + this.FileName + ".xml"))));
				codePlaceholder.Controls.Add(new LiteralControl(String.Format(tabDiv, "Xml")));
			}

			if (_css)
			{
				tabPlaceholder.Controls.Add(new LiteralControl(String.Format(tabDataDiv, "Css", this.Render(this.Path + this.FileName + ".css"))));
				codePlaceholder.Controls.Add(new LiteralControl(String.Format(tabDiv, "Css")));
			}
			if (_cs)
			{
				tabPlaceholder.Controls.Add(new LiteralControl(String.Format(tabDataDiv, "C1S", this.Render(this.Path + this.FileName + ".cs"))));
				codePlaceholder.Controls.Add(new LiteralControl(String.Format(tabDiv, "C1S")));
			}
			if (_ascxCs)
			{
				tabPlaceholder.Controls.Add(new LiteralControl(String.Format(tabDataDiv, "CS", this.Render(this.Path + this.FileName + ".ascx.cs"))));
				codePlaceholder.Controls.Add(new LiteralControl(String.Format(tabDiv, "CS")));
			}

			if (_vb)
			{
				tabPlaceholder.Controls.Add(new LiteralControl(String.Format(tabDataDiv, "VB", this.Render(this.Path + this.FileName + ".ascx.vb"))));
				codePlaceholder.Controls.Add(new LiteralControl(String.Format(tabDiv, "VB")));
			}

			if (this.ImageDir !=string.Empty)
			{
				tabPlaceholder.Controls.Add(new LiteralControl(String.Format(tabDataDiv, "Images", this.RenderImages(this.Path + this.ImageDir))));
				codePlaceholder.Controls.Add(new LiteralControl(String.Format(tabDiv, "Images")));
			}
			tabPlaceholder.Controls.Add(new LiteralControl(string.Format("<script language='JavaScript'>displayTab('{0}');</script>", DefaultTab)));
		}

		private string applicationPath
		{
			get
			{
				if (_applicationPath == "")
				{
					_applicationPath = Page.Request.ApplicationPath;
					if (_applicationPath != "/")
					{
						_applicationPath += "/";
					}
				}
				return _applicationPath;
			}
		}

		string GetLangFromLine(string sourceLine, string defLang)
		{
			if (sourceLine == null)
			{
				return defLang;
			}
			if (ReplaceTabs)
			{
				sourceLine = Regex.Replace(sourceLine, "(?i)(\\t)", TabString);
			}

			Match langMatch = Regex.Match(sourceLine, "(?i)<%@\\s*Page\\s*.*Language\\s*=\\s*\"(?<lang>[^\"]+)\"");
			if (langMatch.Success) 
			{
				return langMatch.Groups["lang"].ToString();
			}

			langMatch = Regex.Match(sourceLine, "(?i)(?=.*runat\\s*=\\s*\"?server\"?)<script.*language\\s*=\\s*\"(?<lang>[^\"]+)\".*>");
			if (langMatch.Success) 
			{
				return langMatch.Groups["lang"].ToString();
			}

			langMatch = Regex.Match(sourceLine, "(?i)<%@\\s*WebService\\s*.*Language\\s*=\\s*\"?(?<lang>[^\"]+)\"?");
			if (langMatch.Success) 
			{
				return langMatch.Groups["lang"].ToString();
			}

			return defLang;
		}

		string FixCSLine(string sourceLine)
		{
			if (sourceLine == null)
				return null;

			if (ReplaceTabs) 
			{
				sourceLine = Regex.Replace(sourceLine, "(?i)(\\t)", TabString);
			}
			sourceLine = HttpUtility.HtmlEncode(sourceLine);

			String[] keywords = new String[] {
												 "private", "protected", "public", "namespace", "class", "break",
												 "for", "if", "else", "while", "switch", "case", "using",
												 "return", "null", "void", "int", "bool", "string", "float",
												 "this", "new", "true", "false", "const", "static", "base",
												 "foreach", "in", "try", "catch", "finally", "get", "set", "char", "default"};

			String CombinedKeywords = "(?<keyword>" + String.Join("|", keywords) + ")";

			sourceLine = Regex.Replace(sourceLine, "\\b" + CombinedKeywords + "\\b(?<!//.*)", TAG_FNTBLUE + "${keyword}" + TAG_EFONT);
			sourceLine = Regex.Replace(sourceLine, "(?<comment>//.*$)", TAG_FNTGRN + "${comment}" + TAG_EFONT);

			return sourceLine;
		}

		string FixJSLine(string sourceLine) 
		{
			if (sourceLine == null)
				return null;

			if (ReplaceTabs) 
			{
				sourceLine = Regex.Replace(sourceLine, "(?i)(\\t)", TabString);
			}
			sourceLine = HttpUtility.HtmlEncode(sourceLine);

			String[] keywords = new String[] {
												 "private", "protected", "public", "namespace", "class", "var",
												 "for", "if", "else", "while", "switch", "case", "using", "get",
												 "return", "null", "void", "int", "string", "float", "this", "set",
												 "new", "true", "false", "const", "static", "package", "function",
												 "internal", "extends", "super", "import", "default", "break", "try",
												 "catch", "finally" };

			String CombinedKeywords = "(?<keyword>" + String.Join("|", keywords) + ")";

			sourceLine = Regex.Replace(sourceLine, "\\b" + CombinedKeywords + "\\b(?<!//.*)", TAG_FNTBLUE + "${keyword}" + TAG_EFONT);
			sourceLine = Regex.Replace(sourceLine, "(?<comment>//.*$)", TAG_FNTGRN + "${comment}" + TAG_EFONT);

			return sourceLine;
		}

		string FixVBLine(string sourceLine) 
		{
			if (sourceLine == null)
				return null;

			if (ReplaceTabs) 
			{
				sourceLine = Regex.Replace(sourceLine, "(?i)(\\t)", TabString);
			}
			sourceLine = HttpUtility.HtmlEncode(sourceLine);

			String[] keywords = new String[] {
												 "Private", "Protected", "Public", "End Namespace", "Namespace",
												 "End Class", "Exit", "Class", "Goto", "Try", "Catch", "End Try",
												 "For", "End If", "If", "Else", "ElseIf", "Next", "While", "And",
												 "Do", "Loop", "Dim", "As", "End Select", "Select", "Case", "Or",
												 "Imports", "Then", "Integer", "Long", "String", "Overloads", "True",
												 "Overrides", "End Property", "End Sub", "End Function", "Sub", "Me",
												 "Function", "End Get", "End Set", "Get", "Friend", "Inherits",
												 "Implements","Return", "Not", "New", "Shared", "Nothing", "Finally",
												 "False", "Me", "My", "MyBase" };

			String CombinedKeywords = "(?<keyword>" + String.Join("|", keywords) + ")";

			sourceLine = Regex.Replace(sourceLine, "(?i)\\b" + CombinedKeywords + "\\b(?<!'.*)", TAG_FNTBLUE + "${keyword}" + TAG_EFONT);
			sourceLine = Regex.Replace(sourceLine, "(?<comment>'(?![^']*&quot;).*$)", TAG_FNTGRN + "${comment}" + TAG_EFONT);

			return sourceLine;
		}

		string FixAspxLine(string sourceLine ) 
		{
			string searchExpr;      // search string
			string replaceExpr;     // replace string

			if ((sourceLine == null) || (sourceLine.Length == 0))
				return sourceLine;

			// Search for \t and replace it with 4 spaces.
			if (ReplaceTabs) 
			{
				sourceLine = Regex.Replace(sourceLine, "(?i)(\\t)", TabString);
			}
			sourceLine = HttpUtility.HtmlEncode(sourceLine);

			// Single line comment or #include references.
			searchExpr = "(?i)(?<a>(^.*))(?<b>(&lt;!--))(?<c>(.*))(?<d>(--&gt;))(?<e>(.*))";
			replaceExpr = "${a}" + TAG_FNTGRN + "${b}${c}${d}" + TAG_EFONT + "${e}";

			if (Regex.IsMatch(sourceLine, searchExpr))
				return Regex.Replace(sourceLine, searchExpr, replaceExpr);

			// Colorize <%@ <type>
			searchExpr = "(?i)" + "(?<a>(&lt;%@))" + "(?<b>(.*))" + "(?<c>(%&gt;))";
			replaceExpr = "<font style='background-color:yellow'>${a}${b}${c}</font>";//<b>${a}${b}${c}</b></font>";

			if (Regex.IsMatch(sourceLine, searchExpr))
				sourceLine = Regex.Replace(sourceLine, searchExpr, replaceExpr);

			// Colorize <%# <type>
			searchExpr = "(?i)" + "(?<a>(&lt;%#))" + "(?<b>(.*))" + "(?<c>(%&gt;))";
			replaceExpr = "${a}" + "<font color=red><b>" + "${b}" + "</b></font>" + "${c}";

			if (Regex.IsMatch(sourceLine, searchExpr))
				sourceLine = Regex.Replace(sourceLine, searchExpr, replaceExpr);

			// Colorize tag <type>
			//(?!/?asp:) - it used to make <asp:> tags blue. Not anymore!
			searchExpr = "(?i)" + "(?<a>(&lt;)(?!%)(?!/?template)(?!/?property)(?!/?ibuyspy:)(/|!)?)" + "(?<b>[^;\\s&]+)" + "(?<c>(\\s|&gt;|\\Z))";
			replaceExpr = "${a}" + TAG_FNTMRN + "${b}" + TAG_EFONT + "${c}";

			if (Regex.IsMatch(sourceLine, searchExpr))
				sourceLine = Regex.Replace(sourceLine, searchExpr, replaceExpr);

			// Colorize asp:|template for runat=server tags <type>
			searchExpr = "(?i)(?<a>&lt;/?)(?<b>(asp:|template|property|IBuySpy:).*)(?<c>&gt;)?";
			replaceExpr = "${a}<font color=blue><b>${b}</b></font>${c}";

			if (Regex.IsMatch(sourceLine, searchExpr))
				sourceLine = Regex.Replace(sourceLine, searchExpr, replaceExpr);

			//colorize begin of tag char(s) "<","</","<%"
			searchExpr = "(?i)(?<a>(&lt;)(/|!)?)";//|%
			replaceExpr = TAG_FNTBLUE + "${a}" + TAG_EFONT;

			if (Regex.IsMatch(sourceLine, searchExpr))
				sourceLine = Regex.Replace(sourceLine, searchExpr, replaceExpr);

			// Colorize end of tag char(s) ">","/>", "%>"
			searchExpr = "(?i)(?<a>(/)?(&gt;))";//|%
			replaceExpr = TAG_FNTBLUE + "${a}" + TAG_EFONT;

			if (Regex.IsMatch(sourceLine, searchExpr))
				sourceLine = Regex.Replace(sourceLine, searchExpr, replaceExpr);

			return sourceLine;
		}

		bool IsScriptBlockTagStart(String source) 
		{
			if (Regex.IsMatch(source, "<script.*runat=\"?server\"?.*>")) 
			{
				return true;
			}
			if (Regex.IsMatch(source, "(?i)<%@\\s*WebService")) 
			{
				return true;
			}
			return false;
		}

		bool IsScriptBlockTagEnd(String source) 
		{
			if (Regex.IsMatch(source, "</script.*>")) 
			{
				return true;
			}
			return false;
		}

		bool IsMultiLineTagStart(String source) 
		{
			String searchExpr = "(?i)(?!.*&gt;)(?<a>&lt;/?)(?<b>(asp:|template|property|IBuySpy:).*)";

			source = HttpUtility.HtmlEncode(source);
			if ( Regex.IsMatch(source, searchExpr) ) 
			{
				return true;
			}
			return false;
		}

		bool IsMultiLineTagEnd(String source) 
		{
			String searchExpr = "(?i)&gt;";

			source = HttpUtility.HtmlEncode(source);
			if ( Regex.IsMatch(source, searchExpr) ) 
			{
				return true;
			}
			return false;
		}

		protected string RenderImages (string DirName){
			try 
			{
				if (DirName != null) 
				{
					DirName = Page.Server.MapPath(DirName.Replace("~/", applicationPath));
					if (!Directory.Exists(DirName)) return "";
					string [] fileEntries = Directory.GetFiles(DirName);															
					string path = this.Path + this.ImageDir + "/";
					string result = string.Empty;
				    foreach(string fileName in fileEntries){
						if ( fileName.EndsWith("gif") || fileName.EndsWith("jpg") || fileName.EndsWith("png") ){
							string name = fileName.Substring(fileName.LastIndexOf("\\") + 1);
							result += "<img src=\"" + path + name +"\" border=1> " + path + name + "<br>";
						}
					}
					return result;
				}
			}catch (Exception e) {
				return e.StackTrace;
			}
			return "";
		}

		protected string RenderHtmlFile (string FileName){
			try 
			{
				if (FileName != null) 
				{
					FileName = Page.Server.MapPath(FileName.Replace("~/", applicationPath));
					FileStream fs = new FileStream(FileName, FileMode.Open, FileAccess.Read);
					StreamReader sr = new StreamReader(fs);
					return sr.ReadToEnd();					
			    }
			}
			catch (Exception e)
			{
				return e.StackTrace;
			}
			return "";
		}

		protected string RenderContent (TextReader sr, string FileName)
		{
			StringWriter textBuffer = new StringWriter();
			String sourceLine;

			if (ReplaceTabs) 
			{
				for (int i=0; i<TabSpaces; i++) 
				{
					TabString += " ";
				}
			}

			if ( FontSize.Value > 12 ) 
			{
				textBuffer.Write("<font><b>\r\n");
			} 
			else 
			{
				textBuffer.Write("<font>\r\n");
			}

			if((FileName.ToLower()).EndsWith(".cs")) 
			{
				textBuffer.Write("<pre style=\"font:" + FontSize.ToString() + " " + FontName + "\">\r\n");
				while((sourceLine = sr.ReadLine()) != null) 
				{
					textBuffer.Write(FixCSLine(sourceLine)) ;
					textBuffer.Write("\r\n");
				}
				textBuffer.Write("</pre>");
			} 
			else if((FileName.ToLower()).EndsWith(".js")) 
			{
				textBuffer.Write("<pre style=\"font:" + FontSize.ToString() + " " + FontName + "\">\r\n");
				while((sourceLine = sr.ReadLine()) != null) 
				{
					textBuffer.Write(FixJSLine(sourceLine)) ;
					textBuffer.Write("\r\n");
				}
				textBuffer.Write("</pre>");
			} 
			else if((FileName.ToLower()).EndsWith(".vb")) 
			{
				textBuffer.Write("<pre style=\"font:" + FontSize.ToString() + " " + FontName + "\">\r\n");
				while((sourceLine = sr.ReadLine()) != null) 
				{
					textBuffer.Write(FixVBLine(sourceLine)) ;
					textBuffer.Write("\r\n");
				}
				textBuffer.Write("</pre>");
			} 
			else 
			{
				String lang = "VB";
				bool isInScriptBlock = false;
				bool isInMultiLine = false;

				textBuffer.Write("<pre style=\"font:" + FontSize.ToString() + " " + FontName + "\">\r\n");
				while((sourceLine = sr.ReadLine()) != null) 
				{
					// First we want to grab the global language
					// for this page by a Page directive.  Or
					// possibly from a script block.
					lang = GetLangFromLine(sourceLine, lang);
					if (IsScriptBlockTagStart(sourceLine)) 
					{
						textBuffer.Write(FixAspxLine(sourceLine));
						isInScriptBlock = true;
					}
					else if (IsScriptBlockTagEnd(sourceLine)) 
					{
						textBuffer.Write(FixAspxLine(sourceLine));
						isInScriptBlock = false;
					}
					else if (IsMultiLineTagStart(sourceLine) && !isInMultiLine) 
					{
						isInMultiLine = true;
						textBuffer.Write("<font color=blue><b>" + HttpUtility.HtmlEncode(sourceLine));
					}
					else if (IsMultiLineTagEnd(sourceLine) && isInMultiLine) 
					{
						isInMultiLine = false;
						textBuffer.Write(HttpUtility.HtmlEncode(sourceLine) + "</b></font>");
					}
					else if (isInMultiLine) 
					{
						textBuffer.Write(HttpUtility.HtmlEncode(sourceLine));
					}
					else 
					{
						if (isInScriptBlock == true) 
						{
							if ( lang.ToLower() == "c#" ) 
							{
								textBuffer.Write(FixCSLine(sourceLine));
							} 
							else if ( lang.ToLower() == "vb" ) 
							{
								textBuffer.Write(FixVBLine(sourceLine));
							} 
							else if ( lang.ToLower() == "jscript" || lang.ToLower() == "javascript" ) 
							{
								textBuffer.Write(FixJSLine(sourceLine));
							}
						}
						else 
						{
							textBuffer.Write(FixAspxLine(sourceLine));
						}
					}
					textBuffer.Write("\r\n");
				}
				textBuffer.Write("</pre>");
			}
			if ( FontSize.Value > 12 ) 
			{
				textBuffer.Write("</b></font>\r\n");
			} 
			else 
			{
				textBuffer.Write("</font>\r\n");
			}

			return(textBuffer.ToString());								
		}

		protected string Render(string FileName) 
		{
			try 
			{
				if (FileName != null) 
				{
					FileName = Page.Server.MapPath(FileName.Replace("~/", applicationPath));
					FileStream fs = new FileStream(FileName, FileMode.Open, FileAccess.Read);
					StreamReader sr = new StreamReader(fs);
					string result = RenderContent(sr, FileName);
					fs.Close();
					return result;
				}
			}
			catch (Exception e) 
			{
				return e.StackTrace;
			}

			return "";
		}
		
		
		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		///		Required method for Designer support - do not modify
		///		the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.Load += new System.EventHandler(this.Page_Load);
			this.PreRender += new System.EventHandler (this.OnPreRender);

		}
		#endregion

		#region Public Properties
		public string DefaultTab
		{
			get
			{
				return _defaultTab;
			}
			set
			{
				_defaultTab = value;
			}
		}

		public bool Ascx
		{
			get
			{
				return _ascx;
			}
			set
			{
				_ascx = value;
			}
		}

		public bool Xml
		{
			get
			{
				return _xml;
			}
			set
			{
				_xml = value;
			}
		}

		public bool Css
		{
			get
			{
				return _css;
			}
			set
			{
				_css = value;
			}
		}

		public bool Cs
		{
			get
			{
				return _cs;
			}
			set
			{
				_cs = value;
			}
		}

		public bool AscxCs
		{
			get
			{
				return _ascxCs;
			}
			set
			{
				_ascxCs = value;
			}
		}

		public bool Vb
		{
			get
			{
				return _vb;
			}
			set
			{
				_vb = value;
			}
		}

		public string Path
		{
			get
			{
				return _path;
			}
			set
			{
				_path = value;
			}
		}

		public bool Description
		{
			get
			{
				return _description;
			}
			set
			{
				_description = value;
			}
		}

		public string ImageDir
		{
			get
			{
				return _imageDir;
			}
			set
			{
				_imageDir = value;
			}
		}


		public string FileName
		{
			get
			{
				return _fileName;
			}
			set
			{
				_fileName = value;
			}
		}


	#endregion

	}
}